<?php
/**
 * My attempt at a Time Handler widget
 */

include_once(dirname(__FILE__)."/../Widget.php");
include_once(dirname(__FILE__)."/../Label.php");


class TimeHandler extends CWCWidget
{
    var $moLabel;
    var $mszTitle = "Select an offset Time";
    var $mszWidgetClass = "";

    /**
     * build a new time handler widget
     */
    function TimeHandler()
    {
        $this->mszLanguageResource = str_replace("\\","/",dirname(__FILE__))."/TimeHandler.dbf";

        // invoke constructor of parent
        parent::CWCWidget();

        $this->mszTitle = trim($this->moMLT->get("0", "Select an Offset Time"));

        $this->mnPriority = PRIORITY_HIGH;

        // set the description for this widget
        $this->szWidgetDescription = <<<EOT
Sends along an offset time in hours to various data sources and returns with data
EOT;

        $this->moLabel = new CWCLabel( $this );

        $this->maAttributes["TITLE"] = new StringAttribute( "TITLE", false );
        $this->maAttributes["WIDGETCLASS"] = new StringAttribute( "WIDGETCLASS", false );
        $this->mnMaturityLevel = MATURITY_BETA;
    }
    
    
    function InitDefaults()
    {
        parent::InitDefaults();
        if (isset($this->maParams["WIDGETCLASS"]))
            $this->mszWidgetClass = strtoupper($this->maParams["WIDGETCLASS"]);
    }

    /**
     * get the javascript functions for this widget
     */
    function GetJavascriptFunctions()
    {
        parent::GetJavascriptFunctions();

        if (isset($this->maSharedResourceWidgets["CWCJSAPI"]))
          $bCWCJSAPI = 1;
        else
          $bCWCJSAPI = 0;

        $aReturn = array();
        $szJsFunctionName = "changeOffset";
        $szFunction = <<<EOT
/**
 * {$szJsFunctionName}
 * called when the Time Offset value is changed.
 */
function {$szJsFunctionName}(obj)
{
    box = document.forms[0].OFFSET;
    offset = box.options[box.selectedIndex].value;
    if (offset != "")
    {
        //alert(offset);
	{$this->mszHTMLForm}.NAV_OFFSET.value=offset;
        {$this->mszHTMLForm}.SEL_OFFSET.value=box.options[box.selectedIndex].text;
        {$this->mszHTMLForm}.NAV_CMD.value = "";
        {$this->mszHTMLForm}.submit();
    }
    
    return;
}
EOT;
        $aReturn[$szJsFunctionName] = $szFunction;
        
        return $aReturn;
    }

    /**
     * get the HTML variables for this widget
     */
   function GetHTMLHiddenVariables()
    {
        parent::GetHTMLHiddenVariables();

        $szVariable = "NAV_OFFSET";
	$szNavOffset = $this->getVar( "NAV_OFFSET" );
        $szValue = " <INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"$szNavOffset\">\n";
        $aReturn[$szVariable] = $szValue;

        $szVariable = "SEL_OFFSET";
	$szSelOffset = $this->getVar( "SEL_OFFSET" );
        $szValue = " <INPUT TYPE=HIDDEN NAME=$szVariable VALUE=\"$szSelOffset\">\n";
        $aReturn[$szVariable] = $szValue;
        
        return $aReturn;
    }    
    
    /**
     * process the url looking for timehandler requests
     */
    function  ParseURL()
    {
        parent::ParseURL();

        if ($this->isVarSet( "NAV_OFFSET" ) &&
            $this->getVar( "NAV_OFFSET" ) != "")
        {
            $szOffset = $this->getVar( "NAV_OFFSET" );

	    $oMap = $this->moMapObject->oMap;
	   
	    //hide mouseovers/insitu currents if offset is not 2
	    
	    //get layers to alter
	    $mouse_layer = $oMap->getLayerByName('Observation MouseOvers');
	    $is_currents_layer = $oMap->getLayerByName('in_situ_currents');
	    
	    if ($szOffset != 2)
	    {
	    // hide MouseOver option in legend
	    $mouse_layer->set('group','Hidden');     		
	    $mouse_layer->SetMetaData('legendvis','hidden');
	    	    
	    //if MouseOvers are ON when the time is changed these will make sure it doesn't render
	    $mouse_layer->set('minscale','10000000');
	    
	    //if is_current are ON when the time is changed these will make sure they don't render
	    $is_currents_layer->set('minscale','10000000');
	    
	    $codar_offset = $szOffset;
	    }
	    
	    else
	    {
            //return MouseOver to legend since -2 hrs
            $mouse_layer->set('group','Observations');
	    $mouse_layer->SetMetaData('legendvis','');
	    
	    //return IS current layers to normal scale dependencies
	    $is_currents_layer->set('minscale','150000');
	    
	    $codar_offset = $szOffset + 2;
	    }
	    

	    //here is the meat of the time offset handling
            $label_layer = $oMap->getLayerByName('request_timestamp');
            $class = $label_layer->getClass(0);
	    
	    //Eastern Time - offset for timestamp labelling
	    $request_time_eastern = time() - ($szOffset * 60 * 60); 
	    
	    //format and send to mapfile for labelling
	    $timestamp = date('m-d-Y g\:\0\0 a', $request_time_eastern);
	    $class->settext($label_layer,"$timestamp");
            
	    //GMT time - offset for DB request
            $request_time = (strtotime (gmdate("m/d/Y H:i:s"))) - ($szOffset * 60 * 60);
            
	    //GMT time - offset and then reformatted for QuikSCAT WMS service
	    $quikscat_time = date('Y_m_d_H_i_\0', $request_time);
	    
	    //GMT time - offset for CODAR DB request
	    $codar_request_time = (strtotime (gmdate("m/d/Y H:i:s"))) - ($codar_offset * 60 * 60);
	    
	    $nLayers = $oMap->numlayers;
            for ($i=0;$i<$nLayers;$i++)
            {
                $oLayer = $oMap->getLayer($i);
		$db_processing = $oLayer->getMetaData('db_processing');
		
                if ($oLayer->connectiontype == MS_WMS && $oLayer->group == 'Observations' && $oLayer->name != 'quikscat')
                {
                    //$con_string = $oLayer->connection;
                    $con_string = $oLayer->GetMetaData("wms_onlineresource");
                    $new_con_string = $con_string.'&time_offset_hours='.$szOffset;
                    //print $new_con_string;
                    $oLayer->set('connection', $new_con_string);
                }
                
		//handling for local QuikSCAT layers		
		if ($oLayer->connectiontype == MS_WMS && $oLayer->name == 'quikscat_wms')
                {
                    $qs_connection = $oLayer->GetMetaData("wms_onlineresource");
                    $new_qs_connection = $qs_connection.'&time='.$quikscat_time;
                    //print $new_qs_connection;
                    $oLayer->set('connection', $new_qs_connection);
                }
		
		if ($oLayer->connectiontype == MS_POSTGIS && $db_processing == '1')
                {
		    //timestamp format is 2006-01-19 16:00 
		    $request_time_db = date('Y-m-d H\:i', $request_time);	
		    $oLayer->setFilter('report_time_stamp = date_trunc(\'hour\',timestamp without time zone \''.$request_time_db.'\')');
		}
		
		if ($oLayer->connectiontype == MS_POSTGIS && $db_processing == '3')
                {
		    //timestamp format is 2006-01-19 16:00 
		    $request_codar_db = date('Y-m-d H\:i', $codar_request_time);	
		    $oLayer->setFilter('report_time_stamp = date_trunc(\'hour\',timestamp without time zone \''.$request_codar_db.'\')');
		}
		
//		if ($oLayer->connectiontype == MS_POSTGIS && $db_processing == '2')
//		{
//		   $request_time_db = date('Y-m-d H\:i', $request_time);
//		   $oLayer->setFilter('((select count(*) from current_prod where (report_time_stamp = date_trunc(\'hour\',timestamp without time zone \''.$request_time_db.'\'))) < 1)');
//		}
				
	    }

	}
        return true;
    }    
    
    /**
     * draw this widget
     */
    function DrawPublish()
    {
        if (!$this->mbVisible)
            return "<!-- Offset is hidden -->";

        if ($this->isVarSet( "SEL_OFFSET" ))
            $szSelOffset = $this->getVar( "SEL_OFFSET" );
        else
            $szSelOffset = "";

        if ($this->mszWidgetClass != "")
            $szWidgetClass = " CLASS=\"".$this->mszWidgetClass."\"";
        else
            $szWidgetClass = "";

        $szResult = "<SELECT NAME=\"OFFSET\" onchange=\"changeOffset(this)\"$szWidgetClass>\n";

        if (isset( $this->maszContents["OFFSET"] ))
        {
            foreach( $this->maszContents["OFFSET"] as $anOffset )
            {
                if (isset($anOffset["HOUR"])) 
                {			
		  $szResult .= "<OPTION VALUE=\"".$anOffset["HOUR"]."\" ".
                        (($szSelOffset==$anOffset["NAME"]) ? "SELECTED":"").
                        ">".$anOffset["NAME"]."</OPTION>\n";
                }
                else
                {
                    $_SESSION['gErrorManager']->setError(ERR_WARNING,
                      trim($this->moMLT->get("3", "ERROR: Invalid HOUR tag definition in TimeHandler.php.")));
                }
            }
        }

        $szResult .= "</SELECT></td>\n";
		    
	   if ($this->isVarSet( "NAV_OFFSET" ) &&
               $this->getVar( "NAV_OFFSET" ) != "")
	    {
	       $timeOffset_end = $this->getVar( "NAV_OFFSET" );
	    }	    
            else
	    {
               $timeOffset_end = 2;
	    }
	    
	    //Eastern Time
	    $EasternTime = time();
	 	    
	    // use offset to build request times	    
	    $request_time_end = $EasternTime - ($timeOffset_end * 60 * 60);
	    $timeOffset_begin = $timeOffset_end + 1;
	    $request_time_begin = $EasternTime - ($timeOffset_begin * 60 * 60);
            
	    //$timestamp_now = date('m-d-Y g\:\0\0 a');
	    $timestamp_end = date('m-d-Y g\:\0\0 a', $request_time_end);
	    $timestamp_end_hour = date('g\:\0\0 a', $request_time_end);
	    
	    $timestamp_begin = date('m-d-Y g\:\3\0 a', $request_time_begin);
	    $timestamp_begin_hour = date('m-d-Y g\:\3\0', $request_time_begin);	    
	    
	    
	// RS Timestamp lookup for labeling - in GMT, out Eastern
	// Site Path
	$site_path = "/opt/inventory/htdocs/";
	
	// Include ADODB library
	require_once($site_path . "/admin/adodb/adodb.inc.php");
	
	// Connection definitions
	$database_type = "postgres7";
	$database_user = "lookup";
	$database_database = "seacoos_test";
	$database_server = "coriolis.marine.unc.edu";
	$database_password = "lookup";

	// takes request time from WMS call above and reformts it
        $request_time = (strtotime (gmdate("m/d/Y H:i:s"))) - ($timeOffset_end * 60 * 60);
	$rs_request_time = date('Y-m-d H:i:\0\0', $request_time);
	
	// Initialize ADODB connection
	$db = ADONewConnection($database_type);
	$db->Connect($database_server, $database_user,$database_password,$database_database);


	// Construct queries - looks for nearest overpass time not older than 48 hrs
	// loop through for quikscat, modis_sst, avhrr

        $rs_layers = array("quikscat","modis_sst","avhrr");
        foreach ($rs_layers as $l)
        {
        $table = $l."_timestamps";

	$query = "select $l from $table"
	   ." where abs(extract(epoch from timestamp without time zone '$rs_request_time')"
	   ." - extract(epoch from $l))"
	   ." = (select min(abs((extract(epoch from timestamp without time zone '$rs_request_time'))"
	   ." - extract(epoch from $l)))"
	   ." from $table)"
	   ." and abs(extract(epoch from timestamp without time zone '$rs_request_time')"
	   ." - extract(epoch from $l))"
	   ." <= 60*60*24*2";
	
	$recordSet = $db->Execute($query);
	if (!$recordSet)
	{
		print $db->ErrorMsg();
	}
	else
	{
		while (!$recordSet->EOF) 
		{
			$db_time = $recordSet->fields[0];
			$recordSet->MoveNext();
		}
	}
	
	$recordSet->Close();
	$rs_timestamp_utc = strtotime($db_time);

        ## need to add Eastern to GMT offset var - hardcoded for now
	$rs_timestamp_eastern = $rs_timestamp_utc - (5*60*60);

        if ($l == "quikscat")
        {
  	$quikscat_time = date('m-d-Y g\:\0\0 a', $rs_timestamp_eastern);
        }
        if ($l == "modis_sst")
        {
        $modis_timestamp = date('m-d-Y g\:i a', $rs_timestamp_eastern);
        }
	if ($l == "avhrr")
        {
        $avhrr_timestamp = date('m-d-Y g\:i a', $rs_timestamp_eastern);
        }
        
        }
	$db->Close();
	    
	    //added b/c CODAR default will be -4 hrs
	    if ($this->getVar( "NAV_OFFSET" ) == "" OR
                $this->getVar( "NAV_OFFSET" ) == "2" )
	    {  
		$codar_time_raw = $EasternTime - (4 * 60 * 60);
		$codar_time = date('m-d-Y g\:\0\0 a', $codar_time_raw);
            }
	    else
	    {
		$codar_time = $timestamp_end;
	    }   
   
	    //return time table
	    //$szResult .="<tr><td>Now: </td><td>".$timestamp_now."</td></tr>\n";
	    $szResult .="<td><b>Data Display Time: </b></td><td><b>".$timestamp_end."</b></td></tr>\n";
	    $szResult .="<tr><td>SST, Water Level, & Wind:</td><td>".$timestamp_begin_hour." - ".$timestamp_end_hour."</td></tr>\n";
	    $szResult .="<tr><td rowspan=\"6\"></td><td>Precipitation RADAR: </td><td>".$timestamp_end."</td></tr>\n";
            $szResult .="<tr><td colspan=\"2\"><hr></td></tr>\n";
	    $szResult .="<tr><td>QuikSCAT winds: </td><td>".$quikscat_time."</td></tr>\n";
	    $szResult .="<tr><td>Surface Currents (CODAR): </td><td>".$codar_time."</td></tr>\n";
	    $szResult .="<tr><td>AVHRR SST: </td><td>".$avhrr_timestamp."</td></tr>\n";
	    $szResult .="<tr><td>MODIS RGB/SST: </td><td>".$modis_timestamp."</td></tr>\n";
	
	
        $szResult = $this->moLabel->DrawPublish( $szResult );
        return $szResult;
    }
}
?>
